/* $Id: x86_compat.h,v 1.24 2002/10/31 21:36:50 mikpe Exp $
 * Performance-monitoring counters driver.
 * x86-specific compatibility definitions for 2.4/2.5 kernels.
 *
 * Copyright (C) 2000-2002  Mikael Pettersson
 */
#include <linux/config.h>
#include <linux/version.h>

/* 2.4.9-ac3 added {read,write}_cr4() macros in <asm-i386/system.h> */
#if !defined(write_cr4)
static inline void write_cr4(unsigned int x)
{
	__asm__ __volatile__("movl %0,%%cr4" : : "r"(x));
}

static inline unsigned int read_cr4(void)
{
	unsigned int x;
	__asm__ __volatile__("movl %%cr4,%0" : "=r"(x));
	return x;
}
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,18)

/* missing from <asm-i386/cpufeature.h> */
#define cpu_has_msr	boot_cpu_has(X86_FEATURE_MSR)

#else /* for 2.4 only, not 2.2 */

/* missing from <asm-i386/processor.h> */
#define cpu_has_mmx	(test_bit(X86_FEATURE_MMX,  boot_cpu_data.x86_capability))
#define cpu_has_msr	(test_bit(X86_FEATURE_MSR,  boot_cpu_data.x86_capability))

#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,8) && !defined(MODULE)
#include <linux/percpu.h>	/* why doesn't this work for modules? */
#else
/* Simulate per_cpu() for older kernels or modular builds. */
#define DEFINE_PER_CPU(type, name) \
	__typeof__(type) name[NR_CPUS] __cacheline_aligned
#define per_cpu(var, cpu)	((var)[(cpu)])
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,23)
#define cpu_online(cpu)		1
#endif

/* irq_enter() and irq_exit() take two parameters in 2.4. However,
   we only use them to disable preemption in the interrupt handler,
   which isn't needed in non-preemptive 2.4 kernels. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#ifdef CONFIG_PREEMPT
#error "not yet ported to 2.4+PREEMPT"
#endif
#undef irq_enter
#undef irq_exit
#define irq_enter()	do{}while(0)
#define irq_exit()	do{}while(0)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#define num_online_cpus()	smp_num_cpus
#endif

extern unsigned long cpu_khz;
