/* $Id: vperfctr.h,v 1.1 2002/10/31 21:36:50 mikpe Exp $
 * Virtual Per-Process Performance-Monitoring Counters driver
 *
 * Copyright (C) 1999-2002  Mikael Pettersson
 */
#ifndef _LINUX_VPERFCTR_H
#define _LINUX_VPERFCTR_H

#include <asm/perfctr.h>

/* user's view of mmap:ed virtual perfctr */
struct vperfctr_state {
	unsigned int magic;
	int si_signo;
	struct perfctr_cpu_state cpu_state;
};

/* `struct vperfctr_state' binary layout version number */
#define VPERFCTR_STATE_MAGIC	0x0201	/* 2.1 */
#define VPERFCTR_MAGIC	((VPERFCTR_STATE_MAGIC<<16)|PERFCTR_CPU_STATE_MAGIC)

/* parameter in VPERFCTR_CONTROL command */
struct vperfctr_control {
	int si_signo;
	struct perfctr_cpu_control cpu_control;
	unsigned long preserve;
};

/*
 * sys_vperfctr(unsigned int cmd, int whom, void *arg)
 *
 *	cmd			   whom, arg
 */
#define VPERFCTR_INFO	 0	/* 0, struct perfctr_cpu_info* */
#define VPERFCTR_SAMPLE	 1	/* fd, NULL */
#define VPERFCTR_UNLINK	 2	/* fd, NULL */
#define VPERFCTR_CONTROL 3	/* fd, struct vperfctr_control* */
#define VPERFCTR_IRESUME 4	/* fd, NULL */
#define VPERFCTR_OPEN	 5	/* pid, NULL */
#define VPERFCTR_CREAT	 6	/* pid, NULL */

#ifdef __KERNEL__

#ifdef CONFIG_PERFCTR_VIRTUAL

struct vperfctr;	/* opaque */

/* process management operations */
extern struct vperfctr *__vperfctr_copy(struct vperfctr*);
extern void __vperfctr_exit(struct vperfctr*);
extern void __vperfctr_suspend(struct vperfctr*);
extern void __vperfctr_resume(struct vperfctr*);
extern void __vperfctr_sample(struct vperfctr*);

static inline void vperfctr_copy_thread(struct thread_struct *thread)
{
	thread->vperfctr = NULL;
}

static inline void vperfctr_exit_thread(struct thread_struct *thread)
{
	struct vperfctr *vperfctr = thread->vperfctr;
	if( vperfctr ) {
		thread->vperfctr = NULL;
		__vperfctr_exit(vperfctr);
	}
}

static inline void vperfctr_suspend_thread(struct thread_struct *prev)
{
	struct vperfctr *vperfctr = prev->vperfctr;
	if( vperfctr )
		__vperfctr_suspend(vperfctr);
}

/* PRE: next is current */
static inline void vperfctr_resume_thread(struct thread_struct *next)
{
	struct vperfctr *vperfctr = next->vperfctr;
	if( vperfctr )
		__vperfctr_resume(vperfctr);
}

static inline void vperfctr_sample_thread(struct thread_struct *thread)
{
#ifdef CONFIG_SMP
	struct vperfctr *vperfctr = thread->vperfctr;
	if( vperfctr )
		__vperfctr_sample(vperfctr);
#endif
}

#else	/* !CONFIG_PERFCTR_VIRTUAL */

static inline void vperfctr_copy_thread(struct thread_struct *t) { }
static inline void vperfctr_exit_thread(struct thread_struct *t) { }
static inline void vperfctr_suspend_thread(struct thread_struct *t) { }
static inline void vperfctr_resume_thread(struct thread_struct *t) { }
static inline void vperfctr_sample_thread(struct thread_struct *t) { }

#endif	/* CONFIG_PERFCTR_VIRTUAL */

#endif	/* __KERNEL__ */

#endif	/* _LINUX_VPERFCTR_H */
